function [L] = calc_L(T, frz)

%{ 
Options for "frz"
	0   = no freezing; use Lv for all temperatures
	0.1 = 0, but with fixed Lv
	1   = freezing happens immediately; step function
	1.1 = 1, but with fixed Lv
	1.2 = freezing happens immediately at -40 C
	2   = gradual freezing; ramp function
	2.1 = 2, but with fixed Lv
	3   = gradual freezing; ramp function (wide)
	3.1 = 3, but with fixed Lv
	4   = ramp from 0 to -20 C
	4.1 = 4, but with fixed Lv
	5   = ramp from 0 to -35 C
	5.1 = 5, but with fixed Lv 
%}

% set freezing effect off by default
if nargin < 2 || isempty(frz), frz = 0; end

% freezing temperature
Ti = 273.15;

% half width of transition (default is delT)
delT = 1;
wide_delT = 3;

% define Ls (sublimation) and Lv (vaporation)
Ls = 2.835e6;
Lv = 2.501 * 10^6 - (4190 - 1870) * (T - 273.15);   % from emanuel (1994), p.115
Lvf = 2.501 * 10^6 - (4190 - 1870) * (15);  % use 15 deg C as fixed Lv

if frz == 0
    L = Lv;

elseif frz == 0.1
    L = Lvf;

% step
elseif frz == 1
    if T < Ti
        L = Ls;
    else
        L = Lv;
    end

elseif frz == 1.1
    if T < Ti
        L = Ls;
    else
        L = Lvf;
    end

elseif frz == 1.2
    if T < Ti - 40
        L = Ls;
    else
        L = Lv;
    end

% ramp
elseif frz == 2  
    if T < Ti - delT
        L = Ls;
    elseif T > Ti + delT
        L = Lv;
    else
        slope = (Lv - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - delT));
    end

elseif frz == 2.1
    if T < Ti - delT
        L = Ls;
    elseif T > Ti + delT
        L = Lvf;
    else
        slope = (Lvf - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - delT));
    end

% wide ramp
elseif frz == 3 
    delT = wide_delT;

    if T < Ti - delT
        L = Ls;
    elseif T > Ti + delT
        L = Lv;
    else
        slope = (Lv - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - delT));
    end

elseif frz == 3.1
    delT = wide_delT;

    if T < Ti - delT
        L = Ls;
    elseif T > Ti + delT
        L = Lvf;
    else
        slope = (Lvf - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - delT));
    end

% shifted ramp
elseif frz == 4 
    delT = 10;

    if T < Ti - 10 - delT
        L = Ls;
    elseif T > Ti -10 + delT
        L = Lv;
    else
        slope = (Lv - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - 10 - delT));
    end

elseif frz == 4.1
    delT = 10;

    if T < Ti - 10 - delT
        L = Ls;
    elseif T > Ti - 10 + delT
        L = Lvf;
    else
        slope = (Lvf - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - 10 - delT));
    end

% shifted ramp
elseif frz == 5 
    delT = 35/2;

    if T < Ti - 35/2 - delT
        L = Ls;
    elseif T > Ti - 35/2 + delT
        L = Lv;
    else
        slope = (Lv - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - 35/2 - delT));
    end

elseif frz == 5.1
    delT = 35/2;

    if T < Ti - 35/2 - delT
        L = Ls;
    elseif T > Ti - 35/2 + delT
        L = Lvf;
    else
        slope = (Lvf - Ls)/(2 * delT);
        L = Ls + slope * (T - (Ti - 35/2 - delT));
    end
end

end
